var ContactListDialog = Class.create(ModalDialog, {

    inputs: null,
    contacts: null,

    _constructor: function(contacts) {
        this.inputs = [];
        this.contacts = contacts;

        this._init();
    },

    getId: function() {
        return "ContactListDialog";
    },

    getSelectedContacts: function() {
        var result = [];

        for (var i = 0, s1 = this.inputs.length; i < s1; ++i) {
            var id = $F(this.inputs[i]);
            for (var j = 0, s2 = this.contacts.length; j < s2; ++j) {
                var contact = this.contacts[j];
                if (contact.id == id) {
                    result.push(contact);
                    break;
                }
            }
        }
        return result;
    },

    setSelectedContacts: function(contacts) {
        for (var i = 0; i < this.inputs.length; ++i) {
            var input = this.inputs[i];
            var found = false;
            for (var j = 0; j < contacts.length; ++j) {
                if (input.value == contacts[j].id) {
                    found = true;
                    break;
                }
            }
            input.checked = found;
        }
    },

    _handleSubmitPressed: function() {
        this.listener.onSubmit(this);
    },

    _handleCancelPressed: function() {
        this.listener.onCancel(this);
    },

    _init: function() {
        this.setTitle(ContactListDialog.TITLE);
        this.setContent(this._createContentPane());
        this.addButton(ModalDialog.BUTTON_OK, ContactListDialog.Buttons.OK, this._handleSubmitPressed.bind(this));
        this.addButton(ModalDialog.BUTTON_CANCEL, ContactListDialog.Buttons.CANCEL, this._handleCancelPressed.bind(this));
    },

    _createContentPane: function() {
        var panel = document.createElement(Tag.DIV);
        for (var i = 0, size = this.contacts.length; i < size; ++i) {
            var row = this._createContactInputRow("contact" + i, this.contacts[i]);
            panel.appendChild(row);
        }
        return panel;
    },

    _createContactInputRow: function(id, contact) {
        var input = Utils.createInput(Tag.INPUT_TYPE_CHECKBOX, id);
        input.id = contact.id;
        input.value = contact.id;
        this.inputs.push(input);

        var label = document.createElement(Tag.LABEL);
        label.htmlFor = input.id;

        var email = document.createElement(Tag.SPAN);
        email.appendChild(document.createTextNode(" <" + contact.email + ">"));
        Utils.setStyle(email, "color:#777;");
        label.appendChild(document.createTextNode(" \"" + contact.name +"\""));
        label.appendChild(email);

        var panel = document.createElement(Tag.PARAGRAPH);
        panel.appendChild(input);
        panel.appendChild(label);

        return panel;
    }

});

ContactListDialog.TITLE = "Contact List";
ContactListDialog.Buttons = {OK: 'OK', CANCEL: 'Cancel'};