var ChangeDirectoryDialog = Class.create(ModalDialog, {

    pathField: null,
    keyboardHandler: null,
    listener: null,

    _constructor: function() {
        this.pathField = Utils.createTextInput("changePathInputField");
        this.pathField.setAttribute("size", 37);

        this.getContent().appendChild(Utils.createLabel(ChangeDirectoryDialog.Labels.PATH, this.pathField.name));
        this.getContent().appendChild(this.pathField);
        this.addButton(ModalDialog.BUTTON_OK, ChangeDirectoryDialog.Buttons.OK, this.onSubmit.bind(this));
        this.addButton(ModalDialog.BUTTON_CANCEL, ChangeDirectoryDialog.Buttons.CANCEL, this.onCancel.bind(this));
        this.setButtonEnabled(ModalDialog.BUTTON_OK, false);
        this.setTitle(ChangeDirectoryDialog.TITLE);
    },

    getId: function() {
        return "ChangeDirectoryDialog";
    },

    show: function(listener) {
        this.listener = Object.extend(ChangeDirectoryDialog.Listener, listener);
        this._attachHandlers();
        this._updateButtons();
        this.setVisible(true);
        this.setSize(380, null);
    },

    hide: function() {
        this._detachHandlers();
        this.dispose();
    },

    getPath: function() {
        var result = $F(this.pathField);
        return (result != null) ? result.replace("\\", "/") : "";
    },

    isCorrect: function() {
        var value = this.getPath();
        return value != null && value.trim().length > 0;
    },

    onSubmit: function() {
        this.listener.onSubmit(this);
    },

    onCancel: function() {
        this.listener.onCancel(this);
    },

    _attachHandlers: function() {
        this.keyboardHandler= this._handleKeyboard.bindAsEventListener(this);
        Event.observe(this.pathField, "keyup", this.keyboardHandler);
    },

    _detachHandlers: function() {
        Event.unobserve(this.pathField, "keyup", this.keyboardHandler);
        this.keyboardHandler = null;
    },

    _handleKeyboard: function(e) {
        this._updateButtons();
    },

    _updateButtons: function() {
        this.setButtonEnabled(ModalDialog.BUTTON_OK, this.isCorrect());
    },

    _requestFocus: function() {
        this._super._requestFocus.call(this, this.pathField);
    }

});

ChangeDirectoryDialog.TITLE = "Change Directory";
ChangeDirectoryDialog.Labels = {PATH: 'Path:'};
ChangeDirectoryDialog.Buttons = {OK: 'Change', CANCEL: 'Cancel'};
ChangeDirectoryDialog.Listener = {
    onSubmit: function(view) {},
    onCancel: function(view) {}
};