var ScriptChannel = Class.create(Channel, {

    url: null,
    scripts: null,
    timers: null,

    _constructor: function(config) {
        this.scripts = [];
        this.timers = [];
    },

    open: function(url, listener) {
        this.url = url;
        this.listener = listener || new Channel.Listener();
    },

    send: function(data, callback) {
        data = data || {};
        var properties = {};
        this._adjustCallbackHandler(data, callback, properties);
        var script = this._createScript(this._createUrl(data), properties);

        this._bindEvents(script, properties);
        this._startTimeoutTimer(script, properties);
        this._enqueueScript(script);
    },

    close: function() {
        while (this.timers.length != 0) {
            this._stopTimeoutTimer(this.timers.shift());
        }
        while (this.scripts.length != 0) {
            this._cleanup(this.scripts.shift());
        }
    },

    _cleanup: function(script) {
        this._unbindEvents(script);

        var ref = window[ScriptChannel._name];
        script.id && ref[script.id] && delete ref[script.id];

        if (script && script.parentNode) {
            script.parentNode.removeChild(script);
        }
    },

    _adjustCallbackHandler: function(data, callback, props) {
        if (Object.isFunction(callback)) {
            var ref = ScriptChannel._name;
            var id = ScriptChannel._generateIdentifier();
            window[ref][id] = function(f) {
                this[id] && delete this[id];
                f.apply(f, Array.prototype.slice.call(arguments, 1));
            }.bind(window[ref], callback);

            data["callback"] = ref + "." + id;
            props.id = id;
        }
    },

    _handleScriptLoaded: function(script, props) {
        if (this._scriptLoaded(script)) {
            this._stopTimeoutTimer(props.timer);
            this._cleanup(script);
            this._dequeueScript(script);
        }
    },

    _startTimeoutTimer: function(script, props) {
        if (this.timeout > 0) {
            props.timer = (function(self, sc) {
                self._handleTimeout(sc);
            }).delay(this.timeout, this, script);
            this.timers.push(props.timer);
        }
    },

    _stopTimeoutTimer: function(timer) {
        timer && window.clearTimeout(timer);
    },

    _handleTimeout: function(script) {
        this._cleanup(script);

        try {
            this.abort();
        } catch (e) {}
        this.listener.onTimeout(this);
    },

    _scriptLoaded: function (script) {
        return !script.readyState || /loaded|complete/.test(script.readyState);
    },

    _bindEvents: function(script, props) {
        script.onload = script.onreadystatechange = this._handleScriptLoaded.bind(this, script, props);
    },

    _unbindEvents: function(script) {
        if (Object.isElement(script)) {
            script.onload = null;
            script.onreadystatechange = null;
        }
    },

    _enqueueScript: function(script) {
        var holder = this._scriptHolder();
        this.scripts.push(holder.insertBefore(script, holder.firstChild));
    },

    _dequeueScript: function(script) {
        var index = this.scripts.indexOf(script);
        if (index != -1) {
            this.scripts[index] = null;
            this.scripts.splice(index, 1);
        }
    },

    _scriptHolder: function() {
        return document.head || document.getElementsByTagName("HEAD")[0] || document.documentElement;
    },

    _createScript: function(url, props) {
        var el = document.createElement("script");
        el.type = "text/javascript";
        el.charset = "UTF-8";
        el.async = "async";
        el.src = url;
        props.id && (el.id = props.id);
        return el;
    },

    _createUrl: function(data) {
        if (this.preventCaching) {
            data["nocache"] = new Date().getTime();
        }

        var uri = encodeURI(this.url);
        var queryString = this._toQueryString(data);
        if (queryString.length > 0) {
            uri += (uri.indexOf("?") == -1 ? "?" : "&") + queryString;
        }

        return uri;
    },

    _toQueryString: function(data) {
        return this._convertToChannelData(data, function(key, value) {
            return key + "=" + encodeURIComponent(value);
        }).join("&");
    }

});
(function() {
    this._name = "__sdc";
    this._sequence = 0;
    this._generateIdentifier = function(){ return "_" + (this._sequence++).toString(36) + new Date().getTime().toString(36)};
    window[this._name] = {};
}).call(ScriptChannel);