var FlashFileChooser = Class.create(FileChooser, {

    id: null,
    flashMovieUrl: null,
    eventQueue: null,
    uploadQueue: null,
    handlers: null,

    _constructor: function(flashMovieUrl) {
        this.id = "flash_uploader_" + new Date().getTime().toString(36);
        this.flashMovieUrl = flashMovieUrl;
        this.eventQueue = [];
    },

    /** override */
    show: function() {
        var ref = window[FlashFileChooser._name];
        try {
            this._super.show.apply(this, arguments);
            ref[this.id] = this._handleEvent.bind(this);
        } catch(e) {
            delete ref[this.id];
            throw e;
        }
    },

    /** override */
    destroy: function() {
        this._super.destroy.apply(this, arguments);
        delete this.handlers;
        delete window[FlashFileChooser._name][this.id];
    },

    /** override */
    asElement: function() {
        return this._flashElement();
    },

    _doCreateUploadQueue: function() {
        return new FlashFileChooser.UploadQueue(this);
    },

    _handleEvent: function(type) {
        if (type) {
            var args = Array.prototype.slice.call(arguments, 1);
            this.eventQueue.push({type: type, args: args});

            (function (self) {
                var item = self.eventQueue.shift();
                var handler = self.handlers[item.type];
                if (Object.isFunction(handler)) {
                    handler.apply(self, item.args);
                }
            }).delay(0, this);
        }
    },

    _handleReady: function() {
        this._super._handleStateChanged.call(this, "complete");
        this._bindEvents();
    },

    _handleFileSelect: function(e) {
        var ids = e.fileIdList || [];
        var items = [];
        var ref = this.asElement();

        for (var i = 0; i < ids.length; ++i) {
            var file = ref.getFile(ids[i]);
            if (file && file.id && file.name) {
                var item = new FileChooser.FileItem(file.name, file.id);
                item.size = file.size || -1;
                items.push(item);
            }
        }

        this.listener.onFileSelect(items);
    },

    _handleUploadProgress: function(e) {
        // do nothing
    },

    _handleUploadComplete: function(e) {
        this.uploadQueue._processUploadComplete(e);
    },

    _handleUploadCompleteData: function(e) {
        this.uploadQueue._processUploadCompleteData(e);
    },

    _handleUploadError: function(e) {
        this.uploadQueue._processUploadError(e);
    },

    /** override */
    _bindEvents: function() {
        this._super._bindEvents.call(this);

        this.handlers = {};
        this.handlers["apiReady"] = this._handleReady.bind(this);
        this.handlers["fileSelect"] = this._handleFileSelect.bind(this);
        this.handlers["uploadProgress"] = this._handleUploadProgress.bind(this);
        this.handlers["uploadComplete"] = this._handleUploadComplete.bind(this);
        this.handlers["uploadCompleteData"] = this._handleUploadCompleteData.bind(this);
        this.handlers["uploadError"] = this._handleUploadError.bind(this);
    },

    /** override */
    _unbindEvents: function() {
        this._super._unbindEvents.call(this);
    },

    _flashElement: function() {
        return $(this.id);
    },

    /** implement */
    _createChooserElement: function() {
        if (Object.isElement(this.chooserElement)) {
            return;
        }

        var flashvars = "callback=" + encodeURIComponent(FlashFileChooser._name + "." + this.id);
        var html = "<object id='" + this.id + "' name='" + this.id + "' data='" + this.flashMovieUrl + "' "
                + "type='application/x-shockwave-flash' width='100%' height='100%'>"
                + "<param name='movie' value='" + this.flashMovieUrl + "'/>"
                + "<param name='allowScriptAccess' value='always'/>"
                + "<param name='wMode' value='transparent'/>"
                + "<param name='quality' value='high'/>"
                + "<param name='menu' value='false'/>"
                + "<param name='flashvars' value='" + flashvars + "'/>";

        var holder = document.createElement("DIV");
        Utils.setStyle(holder, {width: 1, height: 1, position: "absolute", visibility: "visible"/*"hidden"*/});
        holder.innerHTML = html;
        this.chooserElement = this.overflowElement.form.insertBefore(holder, this.overflowElement);

        // http://kb2.adobe.com/cps/400/kb400730.html
        if (Object.isUndefined(window[this.id])) {
            window[this.id] = this._flashElement();
        }
    },

    /** override */
    _destroyChooserElement: function() {
        var flashElement = this._flashElement();
        if (!Object.isElement(flashElement)) {
            this._super._destroyChooserElement.call(this);
            return;
        }

        // DOM/JS IE 6/7 memory leak workaround
        if (typeof flashElement.CallFunction === "unknown") {
            for (var property in flashElement) {
                try {
                    if (Object.isFunction(flashElement[property])) {
                        flashElement[property] = null;
                    }
                } catch (ignored) {}
            }
        }

        // http://kb2.adobe.com/cps/400/kb400730.html
        window[flashElement.id] = null;
        // http://bugs.adobe.com/jira/browse/FP-529
        window["__flash__removeCallback"] = function (instance, name) {
            if (instance != null) {
                instance[name] = null;
            }
        };

        this._super._destroyChooserElement.call(this);
    }
});
(function() {
    this._name = "__ffch";
    // Flash Player Version Detection in AC_RunActiveContent.js
    this.available = window.DetectFlashVer && DetectFlashVer(9, 0, 28);
    window[this._name] = {};
}).call(FlashFileChooser);


FlashFileChooser.UploadQueue = Class.create(FileChooser.UploadQueue, {

    url: null,
    metaData: null,
    flash: null,
    currentItem: null,
    listener: null,
    statusCode: -1,
    waitingForResponse: false,

    _constructor: function(fileChooser) {
        this.flash = fileChooser.asElement();
        this.client = "flash";
    },

    remove: function(fileItem) {
        if (this._super.remove.apply(this, arguments)) {
            return this._invokeMethod("removeFile", fileItem.id);
        }
        return false;
    },

    send: function(url, listener, metaData) {
        this.url = url;
        this.metaData = metaData;
        this.listener = listener || new FileChooser.UploadQueue.Listener();

        this._nextFileUpload();
    },

    abort: function() {
        this.waitingForResponse = false;
        if (this.currentItem) {
            (function(context, method, id) {
                context._invokeMethod.call(context, method, id)
            }).delay(0, this, "cancelUpload", this.currentItem.id);
            this.currentItem = null;
        }
        this._super.abort.call(this);
    },

    _nextFileUpload: function() {
        var item = this.items.pop();
        if (item) {
            this.currentItem = item;
            this._invokeMethod("startUpload", this.url, item.id, this.metaData);
        }
    },

    _processUploadComplete: function(e) {
        this.currentItem = null;
        if (this.items.length) {
            this._nextFileUpload();
        } else {
            this.waitingForResponse = true;
        }
    },

    _processUploadCompleteData: function(e) {
        this.waitingForResponse = false;
        if (this.items.length == 0) {
            this.listener.onComplete();
        }
    },

    _processUploadError: function(e) {
        if (e.statusCode) {
            this.statusCode = e.statusCode;
        } else if (e.error) {
            this.listener.onError();
        }
    },

    _invokeMethod: function(name) {
        var args, fn, ret;
        if (Object.isFunction(this.flash[name])) {
            fn = this.flash[name];
            args = Array.prototype.slice.call(arguments, 1);
            return fn.apply(this.flash, args);
        } else if (Object.isFunction(this.flash.CallFunction)) {
            args = $A(arguments).slice(1);
            var parametersSting = '<invoke name="' + method + '" returntype="javascript">'
                                  + __flash__argumentsToXML(args, 0)
                                  + '</invoke>';
            ret = this.flash.CallFunction(parametersSting);
            return eval(ret);
        } else {
            throw "Unable to invoke flash component."
        }
    }
});