var UploadForm = Class.create({

    url: null,
    files: null,
    forms: null,
    selectedForm: null,
    selectedFormName: null,

    formSelectionDialog: null,

    listener: null,

    _constructor: function(url) {
        this.url = url;
        this.forms = [];
        this.files = [];
    },

    init: function() {
        if (this.formSelectionDialog == null) {
            this.formSelectionDialog = new UploadFormDialog(this.forms, {
                onSubmit: this._handleFormSelectionDone.bind(this),
                onCancel: this._handleFormSelectionCancel.bind(this),
                onCancelUpload: this._handleFormSelectionCancelUpload.bind(this)
            });
        }
    },

    isEmpty: function() {
        return this.forms.length == 0;
    },

    setForms: function(forms) {
        for (var i = 0; i < forms.length; ++i) {
            var form = forms[i];
            this.forms.push({name: form.name, description: form.desc, batch: form.batch});
        }
    },

    start: function(files, listener) {
        this.listener = listener || new UploadForm.Listener();
        this.files = files.clone();

        if (this.isEmpty()) {
            this.listener.onStartUpload();
            return;
        }

        this.init();
        this._displayFormSelectionDialog();
    },

    _displayFormSelectionDialog: function() {
        var viewport = document.viewport.getDimensions();
        this.formSelectionDialog.setSize(viewport.width * 0.4, viewport.height * 0.37);
        this.formSelectionDialog.setTitle(UploadForm.Messages.TITLE);
        this.formSelectionDialog.setVisible(true);
    },

    _displayUploadFormDialog: function() {
        var listener = Object.extend(new FormDialog.Listener(), {
            onLoaded: this._handleUploadFormLoaded.bind(this),
            onSubmit: this._handleUploadFormSubmit.bind(this),
            onCancel: this._handleUploadFormCancel.bind(this),
            onDone: this._handleUploadFormDone.bind(this)
        });

        var dialog = new FormDialog("upload-form-dialog", this.url);
        var viewport = document.viewport.getDimensions();
        dialog.setSize(viewport.width * 0.5, viewport.height * 0.6);
        dialog.show(listener, {form: this.selectedFormName, filename: this.files});
    },

    _handleUploadFormLoaded: function(dialog) {
        this.selectedForm = dialog.getForm();
    },

    _handleUploadFormSubmit: function(dialog) {
        // todo: if (!this._isValidForm(dialog.getForm())) {
        // Event.stop(e);
        // alert(UploadForm.Messages.INVALID_REQUIRED);
        // }
    },

    _handleUploadFormCancel: function(dialog) {
        dialog.dispose();
        this._displayFormSelectionDialog();
    },

    _handleUploadFormDone: function(dialog) {
        dialog.dispose();
        this.listener.onStartUpload();
    },

    _handleFormSelectionDone: function() {
        this.selectedFormName = this.formSelectionDialog.getSelectedItem();
        this.formSelectionDialog.setVisible(false);

        this._displayUploadFormDialog();
    },

    _handleFormSelectionCancel: function() {
        this.formSelectionDialog.setVisible(false);
        this.listener.onStartUpload();
    },

    _handleFormSelectionCancelUpload: function() {
        this.formSelectionDialog.setVisible(false);
        this.listener.onCancelUpload();
    },

    _isValidForm: function(form) {
        for (var i = 0; i < form.elements.length; ++i) {
            var input = form.elements[i];
            var requiredNode = input.getAttributeNode("required");
            var isRequiredField = Tag.INPUT_TYPE_HIDDEN != input.type && (requiredNode && requiredNode.specified);

            if (!isRequiredField) {
                continue;
            }

            if (Utils.isRadioInput(input) && form.elements[input.name].length > 1) {
                var radioGroup = form.elements[input.name];
                if (!this._hasSelectedItem(radioGroup)) {
                    return false;
                }
            } else {
                var value = $F(input) || "";
                if (value.trim().length == 0) {
                    return false;
                }
            }
        }
        return true;
    },

    _hasSelectedItem: function(items) {
        for (var i = 0; i < items.length; ++i) {
            if (items[i].checked) {
                return true;
            }
        }
        return false;
    }
});
UploadForm.Listener = Class.create({
    onStartUpload: function() {},
    onCancelUpload: function() {}
});

UploadForm.Messages = {TITLE: "Form Data", INVALID_REQUIRED: "Please provide all required fields."};
