var UploadView = Class.create({

    addButton: null,
    clearButton: null,
    uploadButton: null,
    container: null,
    fileCounter: null,

    clearButtonHandler: null,
    uploadButtonHandler: null,

    fileList: null,
    fileChooser: null,
    listener: null,

    _constructor: function(fileChooser) {
        this.addButton = $("addfile");
        this.clearButton = $("clearfilelist");
        this.fileCounter = $("filecounter");
        this.uploadButton = $("startupload");
        this.container = $("uploadcontainer");
        this.fileList = new List($("filelist"));
        this.fileChooser = fileChooser;
    },

    show: function(listener) {
        this.listener = listener || new UploadView.Listener();
        Utils.setStyle(this.container, "visibility:visible;height:auto");
        this.fileChooser.show(this.addButton, {
            onReady: this._handleChooserLoaded.bind(this),
            onFileSelect: this._handleFilesChoosed.bind(this)
        });

        this._bindEvents();
        this.update();
    },

    hide: function() {
        this._unbindEvents();
        Utils.setStyle(this.container, "visibility:hidden;height:0");
        this.fileChooser.hide();
        this.fileChooser = null;
        this.listener = null;
    },

    update: function() {
        var count = this.fileList.getItemCount();
        this.fileCounter.innerHTML = count;
        this._setButtonEnabled(this.uploadButton, count != 0);
        this._setButtonEnabled(this.clearButton, this.fileList.getItemCount() != 0);
    },

    getForm: function() {
        return this.uploadButton.form;
    },

    addFile: function(fileItem) {
        this.fileList.addItem(fileItem.filename);
    },

    removeFile: function(file) {
        this.fileList.removeItem(file);
    },

    clear: function() {
        this.fileList.clear();
    },

    /** property change listener */
    propertyChange: function(event) {
        var property = event.propertyName;
        if (List.PCS_SELECTED_INDEX == property || List.PCS_MODEL == property) {
            this.update();
        }
    },

    _setButtonEnabled: function(button, enable) {
        Utils.setEnabled(button, enable, UploadView.Css.BUTTON_DISABLED);
    },

    _handleChooserLoaded: function() {
        this.listener.onReady(this.fileChooser);
    },

    _handleFilesChoosed: function(files) {
        this.listener.onChooseFiles(files);
    },

    _showNoFilesSelectedMessage: function() {
        alert(Uploader.Messages.NO_FILES_SELECTED);
    },

    _showConnectionTimeoutMessage: function() {
        alert(Uploader.Messages.CONNECTION_TIMEOUT);
    },

    _showConnectionErrorMessage: function(message) {
        alert(message || Uploader.Messages.TRANSFER_ERROR);
    },

    _confirmOverwriteMessage: function(files) {
        var message = Uploader.Messages.CONFIRM_OVERWRITE + "\n\n" + files.join("\n");
        return confirm(message);
    },

    _handleClearButtonPressed: function(e) {
        Event.stop(e);
        this.listener.onRemoveAll();
    },

    _handleUploadButtonPressed: function(e) {
        Event.stop(e);
//        this.fileChooser.hide();
        this.listener.onStartUpload();
    },

    _bindEvents: function() {
        this.clearButtonHandler = this._handleClearButtonPressed.bindAsEventListener(this);
        this.uploadButtonHandler = this._handleUploadButtonPressed.bindAsEventListener(this);

        Event.observe(this.clearButton, "click", this.clearButtonHandler);
        Event.observe(this.uploadButton, "click", this.uploadButtonHandler);
        this.fileList.addPropertyChangeListener(this);
    },

    _unbindEvents: function() {
        Event.unobserve(this.clearButton, "click", this.clearButtonHandler);
        Event.unobserve(this.uploadButton, "click", this.uploadButtonHandler);
        this.fileList.removePropertyChangeListener(this);

        this.clearButtonHandler = null;
        this.uploadButtonHandler = null;
    }

});

UploadView.Css = {BUTTON_DISABLED: "disabled"};
UploadView.Listener = Class.create({
    onReady: function(chooser) {},
    onChooseFiles: function(fileItems) {},
    onRemoveAll: function() {},
    onStartUpload: function() {}
});